"use strict";
/**
 * This is a very simple strongly typed event emitter class, see README.md
 * for more details.
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
exports.createPublicEventEmitter = exports.PublicTypedEvent = exports.createEventEmitter = exports.TypedEvent = void 0;
/**
 * A ts-typed-events event to register callbacks on to be invoked when they are
 * emitted.
 */
var TypedEvent = /** @class */ (function () {
    function TypedEvent() {
        /** All the current listeners for this event. */
        this.listeners = [];
    }
    /**
     * Attaches a callback to trigger on all emits for this event.
     *
     * @param callback - The callback to invoke on all emits.
     */
    TypedEvent.prototype.on = function (callback) {
        this.listeners.push({
            once: false,
            callback: callback
        });
    };
    /**
     * Attaches a callback to trigger on only the first emit for this event.
     *
     * This version either takes a callback or returns a promise.
     *
     * @param callback - Optional callback, if specified invokes the callback
     * only once when the event is triggered, then removes it.
     * Otherwise returns a promise that resolves with the value the next time
     * this event is triggered.
     * @returns Nothing if a callback is passed, otherwise a Promise that
     * should resolve once this Event emits.
     */
    TypedEvent.prototype.once = function (callback) {
        var _this = this;
        if (!callback) {
            // then they want us to return the promise
            var promise = new Promise(function (resolve) {
                // this will invoke the version that has a callback,
                // so resolve can be used as the callback
                _this.once(resolve);
            });
            // attach the promise we just made to the listener (it was pushed
            // on the end via this.once() above)
            this.listeners[this.listeners.length - 1].promise = promise;
            return promise;
        }
        // else we were sent a normal callback, so attach it
        this.listeners.push({
            once: true,
            callback: callback
        });
    };
    /**
     * Removes a callback from this event (regardless of once vs on).
     *
     * Returns true if a callback was removed, false otherwise.
     *
     * @param listener - The callback to remove.
     * @returns True if a callback was removed, false otherwise.
     */
    TypedEvent.prototype.off = function (listener) {
        var originalLength = this.listeners.length;
        // remove all listeners that have the same callback as this one
        this.listeners = this.listeners.filter(function (l) {
            return (listener !== l.callback &&
                (!l.promise || listener !== l.promise));
        });
        return this.listeners.length !== originalLength;
    };
    /**
     * Removes ALL callbacks from this event, regardless of once vs on.
     *
     * Returns the number of listeners removed.
     *
     * @returns The number of listeners removed.
     */
    TypedEvent.prototype.offAll = function () {
        var originalLength = this.listeners.length;
        this.listeners.length = 0; // empty our listener array
        return originalLength;
    };
    return TypedEvent;
}());
exports.TypedEvent = TypedEvent;
/**
 * Creates an emitter for an Event.
 *
 * @internal
 * @param event - The event to create an emitter for.
 * @returns A new emitter function for the given event.
 */
function createEmitter(event) {
    // Hack-y, we are reaching into to grab the listeners
    // realistically, this would be a friend style function
    var publicListenersEvent = event;
    /**
     * The emitter function for the event.
     *
     * @param emitting - Whatever is being emitted.
     * @returns True if any listeners were emitted to, false otherwise.
     */
    function emit(emitting) {
        var listeners = publicListenersEvent.listeners;
        var hadListeners = listeners.length > 0;
        for (var _i = 0, listeners_1 = listeners; _i < listeners_1.length; _i++) {
            var listener = listeners_1[_i];
            listener.callback(emitting);
        }
        // remove all listeners that only wanted to listen once
        publicListenersEvent.listeners = listeners.filter(function (_a) {
            var once = _a.once;
            return !once;
        });
        return hadListeners;
    }
    return emit;
}
/** @internal */
function createEmitterWithEvent(event) {
    var emit = createEmitter(event);
    // Hack: Need to find a better way to convince TS this function will have
    // the correct keys on these complex functions.
    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-explicit-any
    var func = emit;
    func.event = event;
    func.emit = emit;
    return func;
}
/**
 * Creates and returns a new Event and the emitter for that Event.
 *
 * @returns A tuple of the [event, emit] both keyed as an array and object.
 */
function createEventEmitter() {
    return createEmitterWithEvent(new TypedEvent());
}
exports.createEventEmitter = createEventEmitter;
/**
 * A specialized Event that holds a reference to its own emit function.
 * This allows any code with access to the Event to also trigger emits.
 */
var PublicTypedEvent = /** @class */ (function (_super) {
    __extends(PublicTypedEvent, _super);
    function PublicTypedEvent() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        /**
         * Emits a value to all the listeners, triggering their callbacks.
         * Returns true if the event had listeners emitted to,
         * false otherwise.
         * Because this exists on the event, any code with access to this event
         * can trigger the callback for all listeners.
         *
         * @param emitting - If the Event has a type, this is the data of that type
         * to emit to all listeners. If no type (undefined) this argument should
         * be omitted.
         * @returns True if the event had listeners emitted to, false otherwise.
         */
        _this.emit = createEmitter(_this);
        return _this;
    }
    return PublicTypedEvent;
}(TypedEvent));
exports.PublicTypedEvent = PublicTypedEvent;
/**
 * A tuple of both [event, emit] and {event, emit},
 * for you to consume however you desire.
 * The emitter can be considered optional, as the PublicEvent returned can self
 * emit. However this is exposed for API parody with the non Public version.
 *
 * @returns A tuple of the [event, emit] both keyed as an array and object.
 */
function createPublicEventEmitter() {
    return createEmitterWithEvent(new PublicTypedEvent());
}
exports.createPublicEventEmitter = createPublicEventEmitter;
